/*****************************************************************************
 * HBarCode1D.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HBarCode1D
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 09:50:21 $
 *
 */

#ifndef H_BAR_CODE_1D_H
#define H_BAR_CODE_1D_H

class LIntExport HBarCode1D: public HRootObject {
public:
  HBarCode1D(void) {id = -1;} 
  HBarCode1D(const HTuple &CodeName, 
             const HTuple &MinCharacters, 
             const HTuple &MaxCharacters);
  HBarCode1D(const char *CodeName, 
             long MinCharacters, 
             long MaxCharacters);
  HBarCode1D(const HTuple &MinCodeLength, 
             const HTuple &MaxCodeLength, 
             const HTuple &ElementSizes, 
             const HTuple &StartElement, 
             const HTuple &StopElement, 
             const HTuple &MaxSizeRatio, 
             const HTuple &DiscreteCode);
  HBarCode1D(long MinCodeLength, 
             long MaxCodeLength, 
             long ElementSizes, 
             long StartElement, 
             long StopElement, 
             double MaxSizeRatio = 2.5, 
             const char *DiscreteCode = "false");

private:
  // Internal Cleanup
  void Cleanup(void);

public:
  // Tools
  const char *ClassName(void) const {return "HBarCode1D";}

  // Cast Operator
  operator HTuple () const {return GetHandle();}

  // Access and Manipulation
  HTuple GetHandle(void) const {return id;}
  void   SetHandle(HTuple ID) {id = ID;}

private:
  HTuple  id;
public:
  // Generate a discrete bar code from the elements widths.
  virtual HTuple Discrete1dBarCode(const HTuple &Elements) const;
  // Decoding of a sequence of elements of a bar code.
  virtual HTuple Decode1dBarCode(const HTuple &BarCodeElements, HTuple *Reference, HTuple *IsCorrect) const;
  virtual HTuple Decode1dBarCode(const HTuple &BarCodeElements, HTuple *Reference, long *IsCorrect) const;
  // Extract the widths of the elements inside a bar code region.
  virtual HTuple Get1dBarCodeScanline(const HImage &BarCodeRegion, const HTuple &GenericName, const HTuple &GenericValue, const HTuple &Orientation, const HTuple &StopIfFound, HTuple *ScanlineNumPoints, HTuple *ScanlineDecoded, HTuple *ScanlinePointsRow, HTuple *ScanlinePointsColumn) const;
  virtual HTuple Get1dBarCodeScanline(const HImage &BarCodeRegion, const HTuple &GenericName, const HTuple &GenericValue, const HTuple &Orientation, const HTuple &StopIfFound, long *ScanlineNumPoints, long *ScanlineDecoded, double *ScanlinePointsRow, double *ScanlinePointsColumn) const;
  // Extract the widths of the elements inside a bar code region.
  virtual HTuple Get1dBarCode(const HImage &BarCodeRegion, const HTuple &GenericName, const HTuple &GenericValue, const HTuple &Orientation) const;
  // Look for multiple bar code regions in an image.
  virtual HRegionArray Find1dBarCodeRegion(const HImage &Image, const HTuple &GenericName, const HTuple &GenericValue, HTuple *Orientation) const;
  virtual HRegionArray Find1dBarCodeRegion(const HImage &Image, const HTuple &GenericName, const HTuple &GenericValue, double *Orientation) const;
  // Look for one bar code in an image.
  virtual HRegion Find1dBarCodeScanline(const HImage &Image, const HTuple &GenericName, const HTuple &GenericValue, const HTuple &StopIfFound, HTuple *BarcodeFound, HTuple *BarCodeElements, HTuple *Orientation, HTuple *ScanlineNumPoints, HTuple *ScanlineDecoded, HTuple *ScanlinePointsRow, HTuple *ScanlinePointsColumn) const;
  virtual HRegion Find1dBarCodeScanline(const HImage &Image, const HTuple &GenericName, const HTuple &GenericValue, const HTuple &StopIfFound, long *BarcodeFound, HTuple *BarCodeElements, double *Orientation, long *ScanlineNumPoints, long *ScanlineDecoded, double *ScanlinePointsRow, double *ScanlinePointsColumn) const;
  // Look for one bar code in an image.
  virtual HRegion Find1dBarCode(const HImage &Image, const HTuple &GenericName, const HTuple &GenericValue, HTuple *BarcodeFound, HTuple *BarCodeElements, HTuple *Orientation) const;
  virtual HRegion Find1dBarCode(const HImage &Image, const HTuple &GenericName, const HTuple &GenericValue, long *BarcodeFound, HTuple *BarCodeElements, double *Orientation) const;
  // Generate a decscription of a 1D bar code.
  virtual void Gen1dBarCodeDescr(const HTuple &CodeName, const HTuple &MinCharacters, const HTuple &MaxCharacters);
  virtual void Gen1dBarCodeDescr(const char *CodeName, long MinCharacters, long MaxCharacters);
  // Generate a generic decscription of a 1D bar code.
  virtual void Gen1dBarCodeDescrGen(const HTuple &MinCodeLength, const HTuple &MaxCodeLength, const HTuple &ElementSizes, const HTuple &StartElement, const HTuple &StopElement, const HTuple &MaxSizeRatio, const HTuple &DiscreteCode);
  virtual void Gen1dBarCodeDescrGen(long MinCodeLength, long MaxCodeLength, long ElementSizes, long StartElement, long StopElement, double MaxSizeRatio, const char *DiscreteCode);
};

#endif
