/*****************************************************************************
 * HComponentTraining.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HComponentTraining
 *
 * (c) 1996-2003 by MVTec Software GmbH
 *                  www.mvtec.com
 * 
 *****************************************************************************
 *
 * $Revision: 1.1 $
 * $Date: 2003/10/07 13:47:48 $
 *
 */

#ifndef H_COMPONENT_TRAINING_H
#define H_COMPONENT_TRAINING_H

class LIntExport HComponentTraining: public HRootObject {
public:
  HComponentTraining(void) {id = -1;}
  HComponentTraining(
          const HImage &ModelImage, 
          const HRegionArray &InitialComponents, 
          const HImageArray &ExampleImages, 
          HRegionArray *ModelComponents, 
          const HTuple &ContrastLow,
          const HTuple &ContrastHigh,
          const HTuple &MinSize,
          const HTuple &MinScore,
          const HTuple &SearchRowTol,
          const HTuple &SearchColumnTol,
          const HTuple &SearchAngleTol,
          const HTuple &TrainingEmphasis,
          const HTuple &AmbiguityCriterion,
          const HTuple &MaxEdgeOverlap,
          const HTuple &ClusterThreshold);
  HComponentTraining(
          const HImage &ModelImage, 
          const HRegionArray &InitialComponents, 
          const HImageArray &ExampleImages, 
          HRegionArray *ModelComponents, 
          const long ContrastLow,
          const long ContrastHigh,
          const long MinSize,
          const double MinScore,
          const long SearchRowTol,
          const long SearchColumnTol,
          const double SearchAngleTol,
          const char *TrainingEmphasis,
          const char *AmbiguityCriterion,
          const double MaxEdgeOverlap,
          const double ClusterThreshold);

  ~HComponentTraining(void);
private:
  // Copying and Assignment not allowed
  HComponentTraining(const HComponentTraining &inst);
  HComponentTraining &operator= (const HComponentTraining &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HComponentTraining";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  //Handle
  long  id;
public:
  virtual long CreateTrainedComponentModel(const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &MinContrastComp, const HTuple &MinScoreComp, const HTuple &NumLevelsComp, const HTuple &AngleStepComp, const HTuple &OptimizationComp, const HTuple &MetricComp, const HTuple &PregenerationComp, HTuple *RootRanking) const;
  virtual long CreateTrainedComponentModel(const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &MinContrastComp, const HTuple &MinScoreComp, const HTuple &NumLevelsComp, const HTuple &AngleStepComp, const HTuple &OptimizationComp, const HTuple &MetricComp, const HTuple &PregenerationComp, long *RootRanking) const;
  virtual long CreateTrainedComponentModel(double AngleStart, double AngleExtent, long MinContrastComp, double MinScoreComp, long NumLevelsComp, double AngleStepComp, const char *OptimizationComp, const char *MetricComp, const char *PregenerationComp, HTuple *RootRanking) const;
  virtual long CreateTrainedComponentModel(double AngleStart, double AngleExtent, long MinContrastComp, double MinScoreComp, long NumLevelsComp, double AngleStepComp, const char *OptimizationComp, const char *MetricComp, const char *PregenerationComp, long *RootRanking) const;
  // Return the relations between the model components that are contained in
  // a training result.
  virtual HRegionArray GetComponentRelations(const HTuple &ReferenceComponent, const HTuple &Image, HTuple *Row, HTuple *Column, HTuple *Phi, HTuple *Length1, HTuple *Length2, HTuple *AngleStart, HTuple *AngleExtent) const;
  virtual HRegionArray GetComponentRelations(const HTuple &ReferenceComponent, const HTuple &Image, double *Row, double *Column, double *Phi, double *Length1, double *Length2, double *AngleStart, double *AngleExtent) const;
  // Return the initial or model components in a certain image.
  virtual HRegionArray GetTrainingComponents(const HTuple &Components, const HTuple &Image, const HTuple &MarkOrientation, HTuple *Row, HTuple *Column, HTuple *Angle, HTuple *Score) const;
  virtual HRegionArray GetTrainingComponents(const HTuple &Components, const HTuple &Image, const HTuple &MarkOrientation, double *Row, double *Column, double *Angle, double *Score) const;
  // Modify the relations within a training result.
  virtual void ModifyComponentRelations(const HTuple &ReferenceComponent, const HTuple &ToleranceComponent, const HTuple &PositionTolerance, const HTuple &AngleTolerance) const;
  // Read a component training result from a file.
  virtual void ReadTrainingComponents(const HTuple &FileName);
  virtual void ReadTrainingComponents(const char *FileName);
  // Write a component training result to a file.
  virtual void WriteTrainingComponents(const HTuple &FileName) const;
  virtual void WriteTrainingComponents(const char *FileName) const;
  // Adopt new parameters that are used to create the model components into the 
  // training result.
  virtual HRegionArray ClusterModelComponents(const HImageArray &TrainingImages, const HTuple &AmbiguityCriterion, const HTuple &MaxContourOverlap, const HTuple &ClusterThreshold) const;
  virtual HRegionArray ClusterModelComponents(const HImageArray &TrainingImages, const char *AmbiguityCriterion, double MaxContourOverlap, double ClusterThreshold) const;
  // Inspect the rigid model components obtained from the training.
  virtual HRegionArray InspectClusteredComponents(const HTuple &AmbiguityCriterion, const HTuple &MaxContourOverlap, const HTuple &ClusterThreshold) const;
  virtual HRegionArray InspectClusteredComponents(const char *AmbiguityCriterion, double MaxContourOverlap, double ClusterThreshold) const;
  // Train components and relations for the component-based matching.
  virtual HRegionArray TrainModelComponents(const HImage &ModelImage, const HRegionArray &InitialComponents, const HImageArray &TrainingImages, const HTuple &ContrastLow, const HTuple &ContrastHigh, const HTuple &MinSize, const HTuple &MinScore, const HTuple &SearchRowTol, const HTuple &SearchColumnTol, const HTuple &SearchAngleTol, const HTuple &TrainingEmphasis, const HTuple &AmbiguityCriterion, const HTuple &MaxContourOverlap, const HTuple &ClusterThreshold);
  virtual HRegionArray TrainModelComponents(const HImage &ModelImage, const HRegionArray &InitialComponents, const HImageArray &TrainingImages, long ContrastLow, long ContrastHigh, long MinSize, double MinScore, long SearchRowTol, long SearchColumnTol, double SearchAngleTol, const char *TrainingEmphasis, const char *AmbiguityCriterion, double MaxContourOverlap, double ClusterThreshold);
};

#endif
