/*****************************************************************************
 * HFunction1D.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HFunction1D
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 09:55:28 $
 *
 */

#ifndef H_FUNCTION_1D_H
#define H_FUNCTION_1D_H

class LIntExport HFunction1D: public HRootObject {
public:
  HFunction1D(void) {id = -1;}
  HFunction1D(const HTuple &YValues);
  HFunction1D(double YValues);
  HFunction1D(const HTuple &XValues, const HTuple &YValues);
  HFunction1D(double XValues, double YValues);

private:
  // Internal Cleanup
  void Cleanup(void);

public:
  // Tools
  const char *ClassName(void) const {return "HFunction1D";}

  // Cast Operator
  operator HTuple () const {return GetHandle();}

  // Access and Manipulation
  HTuple GetHandle(void) const {return id;}
  void   SetHandle(HTuple ID) {id = ID;}

private:
  HTuple  id;
public:
  // Calculate the derivatives of a function.
  virtual HTuple DerivateFunct1d(const HTuple &Mode);
  // Calculate the local minimum and maximum points of a function.
  virtual HTuple LocalMinMaxFunct1d(const HTuple &Mode, const HTuple &Interpolation, HTuple *Max) const;
  // Calculate the zero crossings of a function.
  virtual HTuple ZeroCrossingsFunct1d() const;
  // Multiplication and addition of the y values.
  virtual HTuple ScaleYFunct1d(const HTuple &Mult, const HTuple &Add);
  // Negation of the y values.
  virtual HTuple NegateFunct1d();
  // Absolute value of the y values.
  virtual HTuple AbsFunct1d();
  // Access a function value using the index of the control points.
  virtual HTuple GetPairFunct1d(const HTuple &Index, HTuple *Y) const;
  virtual double GetPairFunct1d(const HTuple &Index, double *Y) const;
  // Number of control points of the function.
  virtual long NumPointsFunct1d() const;
  // Smallest and largest y value of the function.
  virtual double YRangeFunct1d(HTuple *YMax) const;
  virtual double YRangeFunct1d(double *YMax) const;
  // Smallest and largest x value of the function.
  virtual double XRangeFunct1d(HTuple *XMax) const;
  virtual double XRangeFunct1d(double *XMax) const;
  // Access to the x/y values of a function.
  virtual HTuple Funct1dToPairs(HTuple *YValues) const;
  // Sample a function equidistantly in an interval.
  virtual HTuple SampleFunct1d(const HTuple &XMin, const HTuple &XMax, const HTuple &XDist, const HTuple &Border);
  // Transform a function using given transformation parameters.
  virtual HTuple TransformFunct1d(const HTuple &Params);
  // Calculate transformation parameters between two functions.
  virtual HTuple MatchFunct1dTrans(const HTuple &Function2, const HTuple &Border, const HTuple &ParamsConst, const HTuple &UseParams, HTuple *ChiSquare, HTuple *Covar) const;
  virtual HTuple MatchFunct1dTrans(const HTuple &Function2, const HTuple &Border, const HTuple &ParamsConst, const HTuple &UseParams, double *ChiSquare, HTuple *Covar) const;
  // Compute the distance of two functions.
  virtual HTuple DistanceFunct1d(const HTuple &Function2, const HTuple &Mode, const HTuple &Sigma) const;
  // Smooth an equidistant function with a Gaussian function.
  virtual HTuple SmoothFunct1dGauss(const HTuple &Sigma);
  // Compute the positive and negative areas of a function.
  virtual double IntegrateFunct1d(HTuple *Negative) const;
  // Read a function from a file.
  virtual void ReadFunct1d(const HTuple &FileName);
  virtual void ReadFunct1d(const char *FileName);
  // Write a function to a file.
  virtual void WriteFunct1d(const HTuple &FileName) const;
  // Create a function from a sequence of y-values.
  virtual void CreateFunct1dArray(const HTuple &YValues);
  virtual void CreateFunct1dArray(double YValues);
  // Create a function from a set of (x,y) pairs.
  virtual void CreateFunct1dPairs(const HTuple &XValues, const HTuple &YValues);
  virtual void CreateFunct1dPairs(double XValues, double YValues);
  // Smooth a 1D function by averaging its values.
  virtual HTuple SmoothFunct1dMean(const HTuple &SmoothSize, const HTuple &Iterations);
};

#endif
