/*****************************************************************************
 * HMeasure.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HMeasure
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  www.mvtec.com
 * 
 *****************************************************************************
 *
 * $Revision: 1.5 $
 * $Date: 2003/10/02 09:56:38 $
 *
 */

#ifndef H_MEASURE_H
#define H_MEASURE_H

class LIntExport HMeasure: public HRootObject {
public:
  HMeasure(void) {id = -1;}
  HMeasure(
      const HTuple &CenterRow, const HTuple &CenterCol, 
      const HTuple &Radius, 
      const HTuple &AngleStart, const HTuple &AngleExtent, 
      const HTuple &AnnulusRadius, 
      const HTuple &Width, const HTuple &Height, 
      const HTuple &Interpolation);
  HMeasure(
      double CenterRow, double CenterCol, 
      double Radius, 
      double AngleStart, double AngleExtent, 
      double AnnulusRadius, 
      long Width, long Height, 
      const char *Interpolation);
  HMeasure(
      const HTuple &Row, const HTuple &Column, 
      const HTuple &Phi, 
      const HTuple &Length1, const HTuple &Length2, 
      const HTuple &Width, const HTuple &Height, 
      const HTuple &Interpolation);
  HMeasure(
      double Row, double Column, 
      double Phi, 
      double Length1, double Length2, 
      long Width, long Height, 
      const char *Interpolation);

  ~HMeasure(void);
private:
  // Copying and Assignment not allowed
  HMeasure(const HMeasure &inst);
  HMeasure &operator= (const HMeasure &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HMeasure";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  // Handle
  long  id;
public:
  // Extracting points with a particular grey value along a rectangle or an 
  // annular arc.
  virtual HTuple MeasureThresh(const HImage &Image, const HTuple &Sigma, const HTuple &Threshold, const HTuple &Select, HTuple *ColumnThresh, HTuple *Distance) const;
  virtual HTuple MeasureThresh(const HImage &Image, double Sigma, double Threshold, const char *Select, HTuple *ColumnThresh, HTuple *Distance) const;
  // Extract a gray value profile perpendicular to a rectangle or annular arc.
  virtual HTuple MeasureProjection(const HImage &Image) const;
  // Reset a fuzzy member function.
  virtual void ResetFuzzyMeasure(const HTuple &SetType) const;
  virtual void ResetFuzzyMeasure(const char *SetType) const;
  // Specify a normalized fuzzy member function for edge pairs.
  virtual void SetFuzzyMeasureNormPair(const HTuple &PairSize, const HTuple &SetType, const HTuple &Function) const;
  virtual void SetFuzzyMeasureNormPair(double PairSize, const char *SetType, double Function) const;
  // Specify a fuzzy member function.
  virtual void SetFuzzyMeasure(const HTuple &SetType, const HTuple &Function) const;
  virtual void SetFuzzyMeasure(const char *SetType, double Function) const;
  // Extract straight edge pairs perpendicular to a rectangle or an
  // annular arc.
  virtual HTuple FuzzyMeasurePairing(const HImage &Image, const HTuple &Sigma, const HTuple &AmpThresh, const HTuple &FuzzyThresh, const HTuple &Transition, const HTuple &Pairing, const HTuple &NumPairs, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *RowPairCenter, HTuple *ColumnPairCenter, HTuple *FuzzyScore, HTuple *IntraDistance) const;
  virtual HTuple FuzzyMeasurePairing(const HImage &Image, double Sigma, double AmpThresh, double FuzzyThresh, const char *Transition, const char *Pairing, long NumPairs, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *RowPairCenter, HTuple *ColumnPairCenter, HTuple *FuzzyScore, HTuple *IntraDistance) const;
  // Extract straight edge pairs perpendicular to a rectangle or an annular arc.
  virtual HTuple FuzzyMeasurePairs(const HImage &Image, const HTuple &Sigma, const HTuple &AmpThresh, const HTuple &FuzzyThresh, const HTuple &Transition, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *RowEdgeCenter, HTuple *ColumnEdgeCenter, HTuple *FuzzyScore, HTuple *IntraDistance, HTuple *InterDistance) const;
  virtual HTuple FuzzyMeasurePairs(const HImage &Image, double Sigma, double AmpThresh, double FuzzyThresh, const char *Transition, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *RowEdgeCenter, HTuple *ColumnEdgeCenter, HTuple *FuzzyScore, HTuple *IntraDistance, HTuple *InterDistance) const;
  // Extract straight edges perpendicular to a rectangle or an annular arc.
  virtual HTuple FuzzyMeasurePos(const HImage &Image, const HTuple &Sigma, const HTuple &AmpThresh, const HTuple &FuzzyThresh, const HTuple &Transition, HTuple *ColumnEdge, HTuple *Amplitude, HTuple *FuzzyScore, HTuple *Distance) const;
  virtual HTuple FuzzyMeasurePos(const HImage &Image, double Sigma, double AmpThresh, double FuzzyThresh, const char *Transition, HTuple *ColumnEdge, HTuple *Amplitude, HTuple *FuzzyScore, HTuple *Distance) const;
  // Extract straight edge pairs perpendicular to a rectangle or annular arc.
  virtual HTuple MeasurePairs(const HImage &Image, const HTuple &Sigma, const HTuple &Threshold, const HTuple &Transition, const HTuple &Select, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *IntraDistance, HTuple *InterDistance) const;
  virtual HTuple MeasurePairs(const HImage &Image, double Sigma, double Threshold, const char *Transition, const char *Select, HTuple *ColumnEdgeFirst, HTuple *AmplitudeFirst, HTuple *RowEdgeSecond, HTuple *ColumnEdgeSecond, HTuple *AmplitudeSecond, HTuple *IntraDistance, HTuple *InterDistance) const;
  // Extract straight edges perpendicular to a rectangle or annular arc.
  virtual HTuple MeasurePos(const HImage &Image, const HTuple &Sigma, const HTuple &Threshold, const HTuple &Transition, const HTuple &Select, HTuple *ColumnEdge, HTuple *Amplitude, HTuple *Distance) const;
  virtual HTuple MeasurePos(const HImage &Image, double Sigma, double Threshold, const char *Transition, const char *Select, HTuple *ColumnEdge, HTuple *Amplitude, HTuple *Distance) const;
  // Translate a measure object.
  virtual void TranslateMeasure(const HTuple &Row, const HTuple &Column) const;
  virtual void TranslateMeasure(double Row, double Column) const;
  // Prepare the extraction of straight edges perpendicular to an annular arc.
  virtual void GenMeasureArc(const HTuple &CenterRow, const HTuple &CenterCol, const HTuple &Radius, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &AnnulusRadius, const HTuple &Width, const HTuple &Height, const HTuple &Interpolation);
  virtual void GenMeasureArc(double CenterRow, double CenterCol, double Radius, double AngleStart, double AngleExtent, double AnnulusRadius, long Width, long Height, const char *Interpolation);
  // Prepare the extraction of straight edges perpendicular to a rectangle.
  virtual void GenMeasureRectangle2(const HTuple &Row, const HTuple &Column, const HTuple &Phi, const HTuple &Length1, const HTuple &Length2, const HTuple &Width, const HTuple &Height, const HTuple &Interpolation);
  virtual void GenMeasureRectangle2(double Row, double Column, double Phi, double Length1, double Length2, long Width, long Height, const char *Interpolation);
};

#endif
