/*****************************************************************************
 * HPixVal.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Pixeltypes for Halcon-images
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  www.mvtec.com
 * 
 *****************************************************************************
 *
 * $Revision: 1.15 $
 * $Date: 2001/11/07 15:34:16 $
 *
 */


#ifndef H_PIX_VAL_H
#define H_PIX_VAL_H


/****************************************************************************/
/*                               HBasePixVal                                */
/****************************************************************************/
class LIntExport HBasePixVal: public HRootObject  {
public:
  virtual operator HByte(void) const = 0;
  virtual const char *ClassName(void) const = 0;
};



/****************************************************************************/
/*                               HAbstractPixVal                            */
/****************************************************************************/
class LIntExport HAbstractPixVal: public HBasePixVal  {
public:
  virtual operator HByte(void) const = 0;
  virtual operator int(void) const = 0;
  virtual operator long(void) const = 0;
//  virtual operator float(void) const = 0;
  virtual operator double(void) const = 0;
  virtual operator HComplex(void) const = 0;
  virtual const char *ClassName(void) const = 0;
  virtual HAbstractPixVal *Copy(void) const = 0;
  virtual void Delete(void) = 0;
};



/****************************************************************************/
/*                               HIntegerPixVal                             */
/****************************************************************************/
class LIntExport HIntegerPixVal: public HAbstractPixVal {
public:
  HIntegerPixVal(void): val (0) {}
  HIntegerPixVal(HByte Val): val(Val) {}
  HIntegerPixVal(int Val): val(Val) {}
  HIntegerPixVal(long Val): val(Val) {}
  HIntegerPixVal(double Val): val((long)Val) {}
  HIntegerPixVal(HComplex Val): val((long)Val) {}
  HIntegerPixVal &operator = (const HIntegerPixVal &grey)
  { val = grey.val; return *this; }
  ~HIntegerPixVal(void) { Delete(); }

  virtual operator HByte(void) const;
  virtual operator int(void) const;
  virtual operator long(void) const;
//  virtual operator float(void) const  { return (float)val; }
  virtual operator double(void) const;
  virtual operator HComplex(void) const;
  virtual void ToByte(void);
  virtual void ToInt1(void);
  virtual void ToInt2(void);
  virtual void ToUInt2(void);
  virtual const char *ClassName(void) const;
  virtual HAbstractPixVal *Copy(void) const;
  virtual void Delete(void);
private:
  long  val;  
};




/****************************************************************************/
/*                               HByteVal                                   */
/****************************************************************************/
class LIntExport HBytePixVal: public HIntegerPixVal {
public:
  HBytePixVal(void): HIntegerPixVal(0) {}
  HBytePixVal(HByte Val): HIntegerPixVal(Val) {}
  HBytePixVal(int Val): HIntegerPixVal(HClipByte(Val)) {}
  HBytePixVal(long Val): HIntegerPixVal(HClipByte(Val)) {}
//  HBytePixVal(float Val): HIntegerPixVal(HClipByte(Val)) {}
  HBytePixVal(double Val): HIntegerPixVal(HClipByte(Val)) {}
  HBytePixVal(HComplex Val): HIntegerPixVal(HClipByte((long)Val)) {}
  
  const char *ClassName(void) const { return "HByteVal"; }
};



/****************************************************************************/
/*                               HInt1Val                                   */
/****************************************************************************/
class LIntExport HInt1PixVal: public HIntegerPixVal {
public:
  HInt1PixVal(void): HIntegerPixVal(0) {}
  HInt1PixVal(int Val): HIntegerPixVal(HClipInt1(Val)) {}
  HInt1PixVal(long Val): HIntegerPixVal(HClipInt1(Val)) {}
//  HInt1PixVal(float Val): HIntegerPixVal(HClipInt1((long)Val)) {}
  HInt1PixVal(double Val): HIntegerPixVal(HClipInt1((long)Val)) {}
  HInt1PixVal(HComplex Val): HIntegerPixVal(HClipInt1((long)Val)) {}
  
  const char *ClassName(void) const { return "HInt1Val"; }
};



/****************************************************************************/
/*                               HInt2Val                                   */
/****************************************************************************/
class LIntExport HInt2PixVal: public HIntegerPixVal  {
public:
  HInt2PixVal(void): HIntegerPixVal(0) {}
  HInt2PixVal(int Val): HIntegerPixVal(HClipInt2((long)Val)) {}
  HInt2PixVal(long Val): HIntegerPixVal(HClipInt2((long)Val)) {}
//  HInt2PixVal(float Val): HIntegerPixVal(HClipInt2((long)Val)) {}
  HInt2PixVal(double Val): HIntegerPixVal(HClipInt2(Val)) {}
  HInt2PixVal(HComplex Val): HIntegerPixVal(HClipInt2((long)Val)) {}
  
  const char *ClassName(void) const { return "HInt2Val"; }
};


/****************************************************************************/
/*                               HUInt2Val                                  */
/****************************************************************************/
class LIntExport HUInt2PixVal: public HIntegerPixVal  {
public:
  HUInt2PixVal(void): HIntegerPixVal(0) {}
  HUInt2PixVal(int Val): HIntegerPixVal(HClipUInt2((long)Val)) {}
  HUInt2PixVal(long Val): HIntegerPixVal(HClipUInt2((long)Val)) {}
//  HUInt2PixVal(float Val): HIntegerPixVal(HClipUInt2((long)Val)) {}
  HUInt2PixVal(double Val): HIntegerPixVal(HClipUInt2(Val)) {}
  HUInt2PixVal(HComplex Val): HIntegerPixVal(HClipUInt2((long)Val)) {}
  
  const char *ClassName(void) const { return "HUInt2Val"; }
};



/****************************************************************************/
/*                               HInt4Val                                   */
/****************************************************************************/
class LIntExport HInt4PixVal: public HIntegerPixVal  {
public:
  HInt4PixVal(void): HIntegerPixVal(0) {}
  HInt4PixVal(long Val): HIntegerPixVal(Val) {}
  HInt4PixVal(int Val): HIntegerPixVal(Val) {}
//  HInt4PixVal(float Val): HIntegerPixVal(Val) {}
  HInt4PixVal(double Val): HIntegerPixVal(Val) {}
  HInt4PixVal(HComplex Val): HIntegerPixVal((long)Val) {}
  
  const char *ClassName(void) const { return "HInt4Val"; }
};



/****************************************************************************/
/*                               HFloatPixVal                               */
/****************************************************************************/
class LIntExport HFloatPixVal: public HAbstractPixVal  {
public:
  HFloatPixVal(void): val(0.0) {}
  HFloatPixVal(int Val): val(Val) {}
//  HFloatPixVal(float Val): val(Val) {}
  HFloatPixVal(double Val): val(Val) {}
  HFloatPixVal(HComplex Val): val(Val) {}
  ~HFloatPixVal(void) { Delete(); }
  
  operator HByte(void) const { return (HByte)val; }
  operator int(void) const { return (int)val; }
  operator long(void) const { return (long)val; }
//  operator float(void) const { return (float)val; }
  operator double(void) const { return val; }
  operator HComplex(void) const { return HComplex(val); }
  const char *ClassName(void) const { return "HFloatPixVal"; }
  HAbstractPixVal *Copy(void) const;
  void Delete(void);
private:
  double  val;  
};



/****************************************************************************/
/*                               HComplexVal                                */
/****************************************************************************/
class LIntExport HComplexPixVal: public HAbstractPixVal  {
public:
  HComplexPixVal(void): val(0.0) {}
  HComplexPixVal(HByte Val): val(Val) {}
  HComplexPixVal(int Val): val(Val) {}
//  HComplexPixVal(float Val): val(Val) {}
  HComplexPixVal(double Val): val(Val) {}
  HComplexPixVal(HComplex Val): val(Val) {}
  ~HComplexPixVal(void) { Delete(); }
  
  operator HByte(void) const { return (HByte)val; }
  operator int(void) const { return (int)val; }
  operator long(void) const { return (long)val; }
 // operator float(void) const { return (float)val; }
  operator double(void) const { return (double)val; }
  operator HComplex(void) const { return val; }

  const char *ClassName(void) const { return "HComplexPixVal"; }
  HAbstractPixVal *Copy(void) const
  { HComplexPixVal *tmp = new HComplexPixVal((HComplex)val); return tmp; }
  void Delete(void) { val = 0.0; }
private:
   HComplex  val;
};




/****************************************************************************/
/*                               HPixVal                                    */
/****************************************************************************/
class LIntExport HPixVal: public HBasePixVal  {
public:
  HPixVal(void) { val = new HBytePixVal(0); }
  HPixVal(const HComplex &Val) { val = new HComplexPixVal((HComplex)Val); }
  HPixVal(int Val) { val = new HInt4PixVal(Val); }
  HPixVal(long Val) { val = new HInt4PixVal(Val); }
  HPixVal(HByte Val) { val = new HBytePixVal(Val); }
//  HPixVal(float Val) { val = new HFloatPixVal(Val); }
  HPixVal(double Val) { val = new HFloatPixVal(Val); }
  HPixVal(const HPixVal &Val) { val = Val.val->Copy(); }
  virtual ~HPixVal(void) { Delete(); };
  // HPixVal(void *ptr, int type);
  HPixVal &operator = (const HPixVal &grey)
  { Delete(); val = grey.val->Copy(); return *this; }
  
  operator HByte(void) const { return (HByte)(*val); }
  operator int(void) const { return (int)(*val); }
  operator long(void) const { return (long)(*val); }
//  operator float(void) const { return (float)(*val); }
  operator double(void) const { return (double)(*val); }
  operator HComplex(void) const { return (HComplex)(double)(*val); }

  void ToByte(void);
  void ToInt1(void);
  void ToInt2(void);
  void ToUInt2(void);
  void ToInt4(void);
  void ToComplexD(void);
  void ToFloat(void);

  HPixVal & operator ++ (void);
  HPixVal   operator + (const HPixVal &pix) const;
  HPixVal & operator -- (void);
  HPixVal   operator - (const HPixVal &pix) const;
  HPixVal   operator * (const HPixVal &pix) const;
  HPixVal   operator / (const HPixVal &pix) const;

  const char *ClassName(void) const { return "HPixVal"; }
  void Delete(void) { delete val; }
private:
   HAbstractPixVal   *val;
};




#endif
