/*****************************************************************************
 * HRLRegion.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Regions as run length encoding displayed  
 *
 * (c) 1996-2003 by MVTec Software GmbH
 *                  www.mvtec.com
 * 
 *****************************************************************************
 *
 * $Revision: 1.11 $
 * $Date: 2003/07/23 09:53:18 $
 *
 */


#ifndef HRL_REGION_H
#define HRL_REGION_H





/****************************************************************************/
/*                           RunLength                                      */
/****************************************************************************/
class LIntExport RunLength {

private: // vorne wegen inline
  void  Incr(long add);
  void  PrepInsert(void) { if (num >= num_max) Incr(1); }
  void  Sort(void);
  void  Pack(void);
  void  Merge(const RunLength &in);
  HBool  InclPoint(const HDPoint2D &p, long *index) const;    

public:
  RunLength(void): num(0), num_max(0), rl(0), sorted(TRUE) {}
  RunLength(const HDChord &line);
  RunLength(const RunLength &reg);
  RunLength(const HRegion &reg);
  RunLength(long num);
  ~RunLength(void);
  RunLength &operator = (const RunLength &in);
  
  void Difference(const RunLength &subtrahend);
  void Union(const RunLength &in);
  void Intersection(const RunLength &in);
  void Insert(const HDChord &line);
  void Append(const HDChord &chord) 
  { PrepInsert(); 
    rl[num++] = chord; 
  }
  void Insert(const HDPoint2D &p);
  void Insert(const RunLength &r);
  HBool In(const HDPoint2D &p) const;
  void Delete(const HDPoint2D &p);
  HRectangle1 BoundingBox(void) const;
  HBool Equal(const RunLength &comp) const;
  HBool IsEmpty(void) const { return num == 0; }
  long Num(void) const { return num; }
  HDChord & operator [] (long i);
  HDChord const & operator [] (long i) const
  { if ((i < 0) || (i >= num))
    H_EXCEPTION("HDChord const & operator [] (long i) const",
		 "index out of range")
    return rl[i]; }
  void Empty(void);
private:
  long  num;           // Anzahl Sehnen
  long  num_max;       // vorhandener Speicherplatz fuer Sehnen
  HDChord *rl;          // Feld von Sehnen
  HBool  sorted;        // Ist rl sortiert ?
private:
  static float incr;   // Increment von max_num falls Ueberlauf
  static long num_def; // Mindestanzahl von Linien einer Region  
};


/****************************************************************************/
/*                               HRLRegion                                  */
/****************************************************************************/
class LIntExport HRLRegion: public HAbstractRegion {
public:
  HRLRegion(void) {}
  HRLRegion(long n): rl(n) {}
  HRLRegion(const HDChord &line): rl(line) {}
  HRLRegion(const HAbstractRegion *base);
  HRLRegion(const HRegion &reg) : rl(reg) {}
  HRLRegion(const HRLRegion &r): HAbstractRegion(r), rl(r.rl) {}
  HRLRegion &operator = (const HRLRegion &in);
  
  // Tools
  void      Append(const HDChord &line);
  void      Display(const HWindow &w) const;
  HDChord   &operator [] (long i) { return rl[i]; }
  HDChord    operator [] (long i) const { return rl[i]; }
  long      Num(void) const { return rl.Num(); }
  const char     *ClassName(void) const;
  void      Empty(void) { rl.Empty(); }
  HRLRegion &operator /= (const HRLRegion &reg);
  HRLRegion &operator &= (const HRLRegion &reg);
  HRLRegion &operator |= (const HRLRegion &reg);
  HBool      operator == (const HRLRegion &reg);
  
  // virtuelle Basisoperationen aus HAbstractRegion
  HAbstractRegion *CopyAbstractRegion(void) const {
    return new HRLRegion(*this); }
  HAbstractRegion *NewEmpty(void) const { return new HRLRegion(); }
  HRectangle1     BoundingBox(void) const { return rl.BoundingBox(); }
  HBool           In(const HDPoint2D &p) const { return rl.In(p); }
  void           Insert(const HDPoint2D &p) { rl.Insert(p); }
  void           Delete(const HDPoint2D &p) { rl.Delete(p); }
  HBool           IsEmpty(void) const { return rl.IsEmpty(); }
  HAbstractRegion *ToRLRegion(void) const;
  HAbstractRegion *ToHRegion(void) const;
private:
  HRegion GenHRegion(void) const;
  RunLength  rl;  
};

#if !defined(WIN32CE)
extern ostream& operator<<(ostream &s, const HRLRegion &reg);
extern istream& operator>>(istream &s, HRLRegion &reg);
#endif

#endif
