/*****************************************************************************
 * HTemplate.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HTemplate
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 10:00:30 $
 *
 */

#ifndef H_TEMPLATE_H
#define H_TEMPLATE_H

class LIntExport HTemplate: public HRootObject {
public:
  HTemplate(void) {id = -1;}
  HTemplate(
      const HImage &Template, 
      const HTuple &FirstError, 
      const HTuple &NumLevel, 
      const HTuple &Optimize, 
      const HTuple &GrayValues);
  HTemplate(
      const HImage &Template, 
      long          FirstError, 
      long          NumLevel, 
      const char   *Optimize = "sort", 
      const char   *GrayValues = "original");
  HTemplate(
      const HImage &Template, 
      const HTuple &NumLevel, 
      const HTuple &AngleStart, 
      const HTuple &AngleExtend, 
      const HTuple &AngleStep, 
      const HTuple &Optimize, 
      const HTuple &GrayValues);
  HTemplate(
      const HImage &Template, 
      long          NumLevel, 
      double        AngleStart, 
      double        AngleExtend, 
      double        AngleStep, 
      const char   *Optimize = "sort", 
      const char   *GrayValues = "original");

  ~HTemplate(void);
private:
  // Copying and Assignment not allowed
  HTemplate(const HTemplate &inst);
  HTemplate &operator= (const HTemplate &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HTemplate";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  //Handle
  long  id;
public:
  // Preparing a pattern for template matching with rotation.
  virtual void CreateTemplateRot(const HImage &Template, const HTuple &NumLevel, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &AngleStep, const HTuple &Optimize, const HTuple &GrayValues);
  virtual void CreateTemplateRot(const HImage &Template, long NumLevel, double AngleStart, double AngleExtend, double AngleStep, const char *Optimize, const char *GrayValues);
  // Preparing a pattern for template matching.
  virtual void CreateTemplate(const HImage &Template, const HTuple &FirstError, const HTuple &NumLevel, const HTuple &Optimize, const HTuple &GrayValues);
  virtual void CreateTemplate(const HImage &Template, long FirstError, long NumLevel, const char *Optimize, const char *GrayValues);
  // Writing a template to file.
  virtual void WriteTemplate(const HTuple &FileName) const;
  virtual void WriteTemplate(const char *FileName) const;
  // Reading a template from file.
  virtual void ReadTemplate(const HTuple &FileName);
  virtual void ReadTemplate(const char *FileName);
  // Gray value offset for template.
  virtual void SetOffsetTemplate(const HTuple &GrayOffset) const;
  virtual void SetOffsetTemplate(long GrayOffset) const;
  // Define reference position for a matching template.
  virtual void SetReferenceTemplate(const HTuple &Row, const HTuple &Column) const;
  virtual void SetReferenceTemplate(double Row, double Column) const;
  // Adapting a template to the size of an image.
  virtual void AdaptTemplate(const HImageArray &Image) const;
  // Searching all good grayvalue matches in a pyramid.
  virtual HRegionArray FastMatchMg(const HImageArray &Image, const HTuple &MaxError, const HTuple &NumLevel) const;
  virtual HRegionArray FastMatchMg(const HImageArray &Image, double MaxError, long NumLevel) const;
  // Searching the best grayvalue matches in a pre generated pyramid.
  virtual double BestMatchPreMg(const HImageArray &ImagePyramid, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &WhichLevels, HTuple *Column, HTuple *Error) const;
  virtual double BestMatchPreMg(const HImageArray &ImagePyramid, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &WhichLevels, double *Column, double *Error) const;
  virtual double BestMatchPreMg(const HImageArray &ImagePyramid, double MaxError, const char *SubPixel, long NumLevels, long WhichLevels, HTuple *Column, HTuple *Error) const;
  virtual double BestMatchPreMg(const HImageArray &ImagePyramid, double MaxError, const char *SubPixel, long NumLevels, long WhichLevels, double *Column, double *Error) const;
  // Searching the best grayvalue matches in a pyramid.
  virtual double BestMatchMg(const HImageArray &Image, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &WhichLevels, HTuple *Column, HTuple *Error) const;
  virtual double BestMatchMg(const HImageArray &Image, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &WhichLevels, double *Column, double *Error) const;
  virtual double BestMatchMg(const HImageArray &Image, double MaxError, const char *SubPixel, long NumLevels, long WhichLevels, HTuple *Column, HTuple *Error) const;
  virtual double BestMatchMg(const HImageArray &Image, double MaxError, const char *SubPixel, long NumLevels, long WhichLevels, double *Column, double *Error) const;
  // Searching all good matches of a template and an image.
  virtual HRegionArray FastMatch(const HImageArray &Image, const HTuple &MaxError) const;
  virtual HRegionArray FastMatch(const HImageArray &Image, double MaxError) const;
  // Searching the best matching of a template and a pyramid with rotation.
  virtual HTuple BestMatchRotMg(const HImageArray &Image, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, HTuple *Column, HTuple *Angle, HTuple *Error) const;
  virtual double BestMatchRotMg(const HImageArray &Image, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, double *Column, double *Angle, double *Error) const;
  virtual HTuple BestMatchRotMg(const HImageArray &Image, double AngleStart, double AngleExtend, double MaxError, const char *SubPixel, long NumLevels, HTuple *Column, HTuple *Angle, HTuple *Error) const;
  virtual double BestMatchRotMg(const HImageArray &Image, double AngleStart, double AngleExtend, double MaxError, const char *SubPixel, long NumLevels, double *Column, double *Angle, double *Error) const;
  // Searching the best matching of a template and an image with rotation.
  virtual HTuple BestMatchRot(const HImageArray &Image, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &MaxError, const HTuple &SubPixel, HTuple *Column, HTuple *Angle, HTuple *Error) const;
  virtual double BestMatchRot(const HImageArray &Image, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &MaxError, const HTuple &SubPixel, double *Column, double *Angle, double *Error) const;
  virtual HTuple BestMatchRot(const HImageArray &Image, double AngleStart, double AngleExtend, double MaxError, const char *SubPixel, HTuple *Column, HTuple *Angle, HTuple *Error) const;
  virtual double BestMatchRot(const HImageArray &Image, double AngleStart, double AngleExtend, double MaxError, const char *SubPixel, double *Column, double *Angle, double *Error) const;
  // Searching the best matching of a template and an image.
  virtual HTuple BestMatch(const HImageArray &Image, const HTuple &MaxError, const HTuple &SubPixel, HTuple *Column, HTuple *Error) const;
  virtual double BestMatch(const HImageArray &Image, const HTuple &MaxError, const HTuple &SubPixel, double *Column, double *Error) const;
  virtual HTuple BestMatch(const HImageArray &Image, double MaxError, const char *SubPixel, HTuple *Column, HTuple *Error) const;
  virtual double BestMatch(const HImageArray &Image, double MaxError, const char *SubPixel, double *Column, double *Error) const;
};

#endif
