/*****************************************************************************
 * HWindow.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Windows management
 *
 * (c) 1996-2000 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.15 $
 * $Date: 2002/04/23 15:45:27 $
 *
 */

#ifndef H_WINDOW_H
#define H_WINDOW_H

class LIntExport HWindow: public HRootObject {
public:
  HWindow(int Row, int Column, int Width, int Height,
	  const HWindow &Father, const char *Mode = "", 
	  const char *Host = "");
  HWindow(int Row=0, int Column=0, int Width=-1, int Height=-1,
	  int Father = 0, const char *Mode = "", 
	  const char *Host = "", HBool bOpen = TRUE);
  HWindow(const char *mode);
  // constructor for new extern window
  HWindow(const INT* pHWnd, const INT* pHDC, int Row, 
  int Column, int Width, int Height);

  // Destructor
  ~HWindow(void);

  const char *ClassName(void) const {
    return "HBaseWindow";
  }
  // wait for a click with the mouse into the window
  void        Click(void) const;
  // return the HALCON internal handle of the HWindow
  long        WindowHandle(void) const {
    return id;
  }
  // display an arbitrary HObject in the window
  void        Display(const HObject &hObj) {
    hObj.Display(*this);
  }
  // display an arbitrary HObjectArray in the window
  void        Display(const HObjectArray &hObj) {
    hObj.Display(*this);
  }
  HDPoint2D   GetMbutton(int *button) const;
  HDPoint2D   GetMbutton(void) const;
  HDPoint2D   GetMposition(int *button) const;
  HDPoint2D   GetMposition(void) const;
  int         GetButton(void) const;
  HCircle     DrawCircle(void) const;
  HEllipse    DrawEllipse(void) const;
  HRectangle1 DrawRectangle1(void) const;
  HRectangle2 DrawRectangle2(void) const;

private:
  HWindow(const HWindow &w);
  HWindow operator = (const HWindow &w);
  long  id;

public:
  // Fenster oeffnen
  void OpenWindow(int Row=0, int Column=0, int Width=-1, int Height=-1,
                  int Father = 0, const char *Mode = "",
                  const char *Host = "");
  // "externes" Fenster
  void NewExternWindow(const INT* pHWnd, const INT* pHDC, int Row, 
                       int Column, int Width, int Height);
  // set dc of extern Window
  void SetDC( const INT* pHDC);
  // Displays arbitrarily oriented rectangles.
  virtual void DispRectangle2(const HTuple &CenterRow, const HTuple &CenterCol, const HTuple &Phi, const HTuple &Length1, const HTuple &Length2) const;
  virtual void DispRectangle2(double CenterRow, double CenterCol, double Phi, double Length1, double Length2) const;
  // Display of rectangles aligned to the coordinate axes.
  virtual void DispRectangle1(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual void DispRectangle1(double Row1, double Column1, double Row2, double Column2) const;
  // Displays a polyline.
  virtual void DispPolygon(const HTuple &Row, const HTuple &Column) const;
  virtual void DispPolygon(long Row, long Column) const;
  // Draws lines in a window.
  virtual void DispLine(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual void DispLine(double Row1, double Column1, double Row2, double Column2) const;
  // Displays crosses in a window.
  virtual void DispCross(const HTuple &Row, const HTuple &Column, const HTuple &Size, const HTuple &Angle) const;
  virtual void DispCross(double Row, double Column, double Size, double Angle) const;
  // Displays ellipses.
  virtual void DispEllipse(const HTuple &CenterRow, const HTuple &CenterCol, const HTuple &Phi, const HTuple &Radius1, const HTuple &Radius2) const;
  virtual void DispEllipse(long CenterRow, long CenterCol, double Phi, double Radius1, double Radius2) const;
  // Displays a noise distribution.
  virtual void DispDistribution(const HTuple &Distribution, const HTuple &Row, const HTuple &Column, const HTuple &Scale) const;
  virtual void DispDistribution(double Distribution, long Row, long Column, long Scale) const;
  // Displays circles in a window.
  virtual void DispCircle(const HTuple &Row, const HTuple &Column, const HTuple &Radius) const;
  virtual void DispCircle(double Row, double Column, double Radius) const;
  // Displays arrows in a window.
  virtual void DispArrow(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2, const HTuple &Size) const;
  virtual void DispArrow(double Row1, double Column1, double Row2, double Column2, double Size) const;
  // Displays circular arcs in a window.
  virtual void DispArc(const HTuple &CenterRow, const HTuple &CenterCol, const HTuple &Angle, const HTuple &BeginRow, const HTuple &BeginCol) const;
  virtual void DispArc(double CenterRow, double CenterCol, double Angle, long BeginRow, long BeginCol) const;
  // Get the operating system window handle.
  virtual long GetOsWindowHandle(HTuple *OSDisplayHandle) const;
  // Set the device context of a virtual graphics window (Windows NT).
  virtual void SetWindowDc(const HTuple &WINHDC) const;
  virtual void SetWindowDc(long WINHDC) const;
  // Create a virtual graphics window under Windows NT.
  virtual long NewExternWindow(const HTuple &WINHWnd, const HTuple &WINHDC, const HTuple &Row, const HTuple &Column, const HTuple &Width, const HTuple &Height) const;
  virtual long NewExternWindow(long WINHWnd, long WINHDC, long Row, long Column, long Width, long Height) const;
  // Interactive output from two window buffers.
  virtual void SlideImage(const HTuple &WindowHandleSource2, const HTuple &WindowHandle) const;
  virtual void SlideImage(long WindowHandleSource2, long WindowHandle) const;
  // Modify position and size of a window.
  virtual void SetWindowExtents(const HTuple &Row, const HTuple &Column, const HTuple &Width, const HTuple &Height) const;
  virtual void SetWindowExtents(long Row, long Column, long Width, long Height) const;
  // Open a graphics window.
  virtual long OpenWindow(const HTuple &Row, const HTuple &Column, const HTuple &Width, const HTuple &Height, const HTuple &FatherWindow, const HTuple &Mode, const HTuple &Machine) const;
  virtual long OpenWindow(long Row, long Column, long Width, long Height, long FatherWindow, const char *Mode, const char *Machine) const;
  // Open a textual window.
  virtual long OpenTextwindow(const HTuple &Row, const HTuple &Column, const HTuple &Width, const HTuple &Height, const HTuple &BorderWidth, const HTuple &BorderColor, const HTuple &BackgroundColor, const HTuple &FatherWindow, const HTuple &Mode, const HTuple &Machine) const;
  virtual long OpenTextwindow(long Row, long Column, long Width, long Height, long BorderWidth, const char *BorderColor, const char *BackgroundColor, long FatherWindow, const char *Mode, const char *Machine) const;
  // Copy inside an output window.
  virtual void MoveRectangle(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2, const HTuple &DestRow, const HTuple &DestColumn) const;
  virtual void MoveRectangle(long Row1, long Column1, long Row2, long Column2, long DestRow, long DestColumn) const;
  // Get the window type.
  virtual HTuple GetWindowType() const;
  // Access to a window's pixel data.
  virtual long GetWindowPointer3(HTuple *ImageGreen, HTuple *ImageBlue, HTuple *Width, HTuple *Height) const;
  // Information about a window's size and position.
  virtual long GetWindowExtents(HTuple *Column, HTuple *Width, HTuple *Height) const;
  // Write the window content to a file.
  virtual void DumpWindow(const HTuple &Device, const HTuple &FileName) const;
  // Copy all pixels within rectangles between output windows.
  virtual void CopyRectangle(const HTuple &WindowHandleDestination, const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2, const HTuple &DestRow, const HTuple &DestColumn) const;
  virtual void CopyRectangle(long WindowHandleDestination, long Row1, long Column1, long Row2, long Column2, long DestRow, long DestColumn) const;
  // Close an output window.
  virtual void CloseWindow() const;
  // Delete an output window.
  virtual void ClearWindow() const;
  // Delete a rectangle on the output window.
  virtual void ClearRectangle(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual void ClearRectangle(long Row1, long Column1, long Row2, long Column2) const;
  // Write look-up-table (lut) as file.
  virtual void WriteLut(const HTuple &FileName) const;
  virtual void WriteLut(const char *FileName) const;
  // Graphical view of the look-up-table (lut).
  virtual void DispLut(const HTuple &Row, const HTuple &Column, const HTuple &Scale) const;
  virtual void DispLut(long Row, long Column, long Scale) const;
  // Query all available look-up-tables (lut).
  virtual HTuple QueryLut() const;
  // Manipulate look-up-table (lut) interactively.
  virtual void DrawLut() const;
  // Get modification parameters of look-up-table (lut).
  virtual double GetLutStyle(HTuple *Saturation, HTuple *Intensity) const;
  // Changing the look-up-table (lut).
  virtual void SetLutStyle(const HTuple &Hue, const HTuple &Saturation, const HTuple &Intensity) const;
  virtual void SetLutStyle(double Hue, double Saturation, double Intensity) const;
  // Get current look-up-table (lut).
  virtual HTuple GetLut() const;
  // Set "look-up-table" (lut).
  virtual void SetLut(const HTuple &LookUpTable) const;
  // Get mode of fixing of current look-up-table (lut).
  virtual HTuple GetFix() const;
  // Set fixing of "look-up-table" (lut)
  virtual void SetFix(const HTuple &Mode) const;
  virtual void SetFix(const char *Mode) const;
  // Get fixing of "look-up-table" (lut) for "real color images"
  virtual HTuple GetFixedLut() const;
  // Fix "look-up-table" (lut) for "real color images".
  virtual void SetFixedLut(const HTuple &Mode) const;
  virtual void SetFixedLut(const char *Mode) const;
  // Set the current mouse pointer shape.
  virtual void SetMshape(const HTuple &Cursor) const;
  virtual void SetMshape(const char *Cursor) const;
  // Query the current mouse pointer shape.
  virtual HTuple GetMshape() const;
  // Query all available mouse pointer shapes.
  virtual HTuple QueryMshape() const;
  // Query the mouse position.
  virtual long GetMposition(HTuple *Column, HTuple *Button) const;
  // Wait until a mouse button is pressed.
  virtual long GetMbutton(HTuple *Column, HTuple *Button) const;
  // Define the region output shape.
  virtual void SetShape(const HTuple &Shape) const;
  virtual void SetShape(const char *Shape) const;
  // Set the color definition via RGB values.
  virtual void SetRgb(const HTuple &Red, const HTuple &Green, const HTuple &Blue) const;
  virtual void SetRgb(long Red, long Green, long Blue) const;
  // Define a color lookup table index.
  virtual void SetPixel(const HTuple &Pixel) const;
  virtual void SetPixel(long Pixel) const;
  // Define an interpolation method for grayvalue output.
  // 
  virtual void SetPartStyle(const HTuple &Style) const;
  virtual void SetPartStyle(long Style) const;
  // Modify the displayed image part.
  virtual void SetPart(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual void SetPart(long Row1, long Column1, long Row2, long Column2) const;
  // Define the grayvalue output mode.
  virtual void SetPaint(const HTuple &Mode) const;
  // Define the line width for region contour output.
  virtual void SetLineWidth(const HTuple &Width) const;
  virtual void SetLineWidth(long Width) const;
  // Define a contour output pattern.
  virtual void SetLineStyle(const HTuple &Style) const;
  virtual void SetLineStyle(long Style) const;
  // Define the approximation error for contour display.
  virtual void SetLineApprox(const HTuple &Approximation) const;
  virtual void SetLineApprox(long Approximation) const;
  // Define the pixel output function.
  virtual void SetInsert(const HTuple &Mode) const;
  virtual void SetInsert(const char *Mode) const;
  // Define output colors (HSI-coded).
  virtual void SetHsi(const HTuple &Hue, const HTuple &Saturation, const HTuple &Intensity) const;
  virtual void SetHsi(long Hue, long Saturation, long Intensity) const;
  // Define grayvalues for region output.
  virtual void SetGray(const HTuple &GrayValues) const;
  virtual void SetGray(long GrayValues) const;
  // Define the region fill mode.
  virtual void SetDraw(const HTuple &Mode) const;
  virtual void SetDraw(const char *Mode) const;
  // Define the image matrix output clipping.
  virtual void SetComprise(const HTuple &Mode) const;
  virtual void SetComprise(const char *Mode) const;
  // Set multiple output colors.
  virtual void SetColored(const HTuple &NumberOfColors) const;
  virtual void SetColored(long NumberOfColors) const;
  // Set output color.
  virtual void SetColor(const HTuple &Color) const;
  virtual void SetColor(const char *Color) const;
  // Get the current region output shape.
  virtual HTuple GetShape() const;
  // Get the current color in RGB-coding.
  virtual HTuple GetRgb(HTuple *Green, HTuple *Blue) const;
  // Get the current color lookup table index.
  virtual HTuple GetPixel() const;
  // Get the current interpolation mode for grayvalue display.
  virtual long GetPartStyle() const;
  // Get the image part.
  virtual long GetPart(HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  // Get the current display mode for grayvalues.
  virtual HTuple GetPaint() const;
  // Get the current line width for contour display.
  virtual long GetLineWidth() const;
  // Get the current graphic mode for contours.
  virtual HTuple GetLineStyle() const;
  // Get the current approximation error for contour display.
  virtual long GetLineApprox() const;
  // Get the current display mode.
  virtual HTuple GetInsert() const;
  // Get the HSI coding of the current color.
  virtual HTuple GetHsi(HTuple *Saturation, HTuple *Intensity) const;
  // Get the current region fill mode.
  virtual HTuple GetDraw() const;
  // Query the grayvalue display modes.
  virtual HTuple QueryPaint() const;
  // Query the possible graphic modes.
  virtual HTuple QueryInsert() const;
  // Query the displayable grayvalues.
  virtual HTuple QueryGray() const;
  // Query all color names.
  virtual HTuple QueryAllColors() const;
  // Query all color names displayable in the window.
  virtual HTuple QueryColor() const;
  // Print text in a window.
  virtual void WriteString(const HTuple &String) const;
  // Set the shape of the text cursor.
  virtual void SetTshape(const HTuple &TextCursor) const;
  virtual void SetTshape(const char *TextCursor) const;
  // Set the position of the text cursor.
  virtual void SetTposition(const HTuple &Row, const HTuple &Column) const;
  virtual void SetTposition(long Row, long Column) const;
  // Read a string in a text window.
  virtual HTuple ReadString(const HTuple &InString, const HTuple &Length) const;
  virtual HTuple ReadString(const char *InString, long Length) const;
  // Read a character from a text window.
  virtual HTuple ReadChar(HTuple *Code) const;
  // Set the position of the text cursor to the beginning of the next line.
  virtual void NewLine() const;
  // Get the shape of the text cursor.
  virtual HTuple GetTshape() const;
  // Get cursor position.
  virtual long GetTposition(HTuple *Column) const;
  // Get the spatial size of a string.
  virtual long GetStringExtents(const HTuple &Values, HTuple *Descent, HTuple *Width, HTuple *Height) const;
  // Query the available fonts.
  virtual HTuple QueryFont() const;
  // Query all shapes available for text cursors.
  virtual HTuple QueryTshape() const;
  // Set the font used for text output.
  virtual void SetFont(const HTuple &Font) const;
  virtual void SetFont(const char *Font) const;
  // Get the current font.
  virtual HTuple GetFont() const;
  // Interactive drawing of a contour.
  virtual HXLDCont DrawXld(const HTuple &Rotate, const HTuple &Move, const HTuple &Scale, const HTuple &KeepRatio) const;
  virtual HXLDCont DrawXld(const char *Rotate, const char *Move, const char *Scale, const char *KeepRatio) const;
  // Interactive drawing of any orientated rectangle.
  virtual double DrawRectangle2Mod(const HTuple &RowIn, const HTuple &ColumnIn, const HTuple &PhiIn, const HTuple &Length1In, const HTuple &Length2In, HTuple *Column, HTuple *Phi, HTuple *Length1, HTuple *Length2) const;
  virtual double DrawRectangle2Mod(double RowIn, double ColumnIn, double PhiIn, double Length1In, double Length2In, HTuple *Column, HTuple *Phi, HTuple *Length1, HTuple *Length2) const;
  // Interactive drawing of any orientated rectangle.
  virtual double DrawRectangle2(HTuple *Column, HTuple *Phi, HTuple *Length1, HTuple *Length2) const;
  // Draw a rectangle parallel to the coordinate axis.
  virtual double DrawRectangle1Mod(const HTuple &Row1In, const HTuple &Column1In, const HTuple &Row2In, const HTuple &Column2In, HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  virtual double DrawRectangle1Mod(double Row1In, double Column1In, double Row2In, double Column2In, HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  // Draw a rectangle parallel to the coordinate axis.
  virtual double DrawRectangle1(HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  // Draw a point.
  virtual double DrawPointMod(const HTuple &RowIn, const HTuple &ColumnIn, HTuple *Column) const;
  virtual double DrawPointMod(double RowIn, double ColumnIn, HTuple *Column) const;
  // Draw a point.
  virtual double DrawPoint(HTuple *Column) const;
  // Draw a line.
  virtual double DrawLineMod(const HTuple &Row1In, const HTuple &Column1In, const HTuple &Row2In, const HTuple &Column2In, HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  virtual double DrawLineMod(double Row1In, double Column1In, double Row2In, double Column2In, HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  // Draw a line.
  virtual double DrawLine(HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  // Interactive drawing of an ellipse.
  virtual double DrawEllipseMod(const HTuple &RowIn, const HTuple &ColumnIn, const HTuple &PhiIn, const HTuple &Radius1In, const HTuple &Radius2In, HTuple *Column, HTuple *Phi, HTuple *Radius1, HTuple *Radius2) const;
  virtual double DrawEllipseMod(double RowIn, double ColumnIn, double PhiIn, double Radius1In, double Radius2In, HTuple *Column, HTuple *Phi, HTuple *Radius1, HTuple *Radius2) const;
  // Interactive drawing of an ellipse.
  virtual double DrawEllipse(HTuple *Column, HTuple *Phi, HTuple *Radius1, HTuple *Radius2) const;
  // Interactive drawing of a circle.
  virtual double DrawCircleMod(const HTuple &RowIn, const HTuple &ColumnIn, const HTuple &RadiusIn, HTuple *Column, HTuple *Radius) const;
  virtual double DrawCircleMod(double RowIn, double ColumnIn, double RadiusIn, HTuple *Column, HTuple *Radius) const;
  // Interactive drawing of a circle.
  virtual double DrawCircle(HTuple *Column, HTuple *Radius) const;
  // Interactive drawing of a closed region.
  virtual HRegion DrawRegion() const;
  // Interactive drawing of a polygon row.
  virtual HRegion DrawPolygon() const;
  // Project and visualize the 3D model of the calibration table in the image.
  virtual void DispCaltab(const HTuple &CalTabDescrFile, const HTuple &CamParam, const HTuple &CamPose, const HTuple &ScaleFac) const;
  virtual void DispCaltab(const char *CalTabDescrFile, double CamParam, double CamPose, double ScaleFac) const;
};

#endif
