/*****************************************************************************
 * HXLDParaArray.h
 ***************************************************************************** 
 *
 * Project:     Halcon/libhalcon
 * Description: Class HXLDParaArray
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.13 $
 * $Date: 2001/11/22 13:46:55 $
 *
 */

#ifndef XLD_PARA_ARRAY_H
#define XLD_PARA_ARRAY_H


/****************************************************************************/
/*                               HXLDParaArray                              */
/****************************************************************************/

class LIntExport HXLDParaArray: public HXLDArray {
public:
  HXLDParaArray(void) : HXLDArray() {num_xld_para_arrays++;}
  HXLDParaArray(Hkey *key, long num);
  HXLDParaArray(const Hobject &obj);
  HXLDParaArray(const HXLDPara &para);
  HXLDParaArray(const HXLD &xld);
  HXLDParaArray(const HXLDParaArray &arr);
  HXLDParaArray(const HXLDArray &arr);
  ~HXLDParaArray(void) {num_xld_para_arrays--;}

  HXLDParaArray &operator= (const HXLDParaArray &arr);
  HXLDParaArray &operator= (const HXLDArray &arr);

  // Tools
  virtual const char *HClassName(void)    const;
  virtual const char *ClassName(void)     const;
  virtual const char *InstClassName(void) const;
  virtual       INT4  ClassDBID(void)     const {return XLD_PARALLEL_ID;}
  virtual       Hkey  Key(long i)         const {return array[i].Key();}
  virtual    Hobject  ElemId(long i)      const {return array[i].Id();}

  // Access and Manipulation
  HXLDPara  const   &operator[] (long index) const;
  HXLDPara          &operator[] (long index);
  HXLDParaArray      operator() (long min, long max) const;
  HXLDParaArray     &Append(const HXLDPara &para);
  HXLDParaArray     &Append(const HXLD &xld);
  HXLDParaArray     &Append(const HXLDParaArray &paras);
  HXLDParaArray     &Append(const HXLDArray &xlds);
  virtual void       SetKeys(Hkey *key, long num);
  void               CopyHXLDParaArray(const HXLDParaArray &in);
  void               CopyHXLDArray(const HXLDArray &in);

  // Class Functions
  static long NumInst(void) {return num_xld_para_arrays;}

private:
  static long   num_xld_para_arrays; 
  void          NewLength(long new_num, long old_num);

public:
  // Extract parallel XLD polygons enclosing a homogeneous area.
  virtual HXLDModParaArray ModParallelsXld(const HImage &Image, HXLDExtParaArray *ExtParallels, const HTuple &Quality, const HTuple &MinGray, const HTuple &MaxGray, const HTuple &MaxStandard) const;
  virtual HXLDModParaArray ModParallelsXld(const HImage &Image, HXLDExtParaArray *ExtParallels, double Quality, long MinGray, long MaxGray, double MaxStandard) const;
  // Return information about the gray values of the area enclosed by XLD
  // parallels.
  virtual double InfoParallelsXld(const HImage &Image, HTuple *QualityMax, HTuple *GrayMin, HTuple *GrayMax, HTuple *StandardMin, HTuple *StandardMax) const;
};

#endif
