/*****************************************************************************
 * HXLDPolyArray.h
 ***************************************************************************** 
 *
 * Projekt:     Halcon/libhalcon
 * Description: Class HXLDPolyArray
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.12 $
 * $Date: 2001/11/22 13:46:55 $
 *
 */

#ifndef XLD_POLY_ARRAY_H
#define XLD_POLY_ARRAY_H

/****************************************************************************/
/*                               HXLDPolyArray                              */
/****************************************************************************/

class LIntExport HXLDPolyArray: public HXLDArray {
public:
  HXLDPolyArray(void) : HXLDArray() {num_xld_poly_arrays++;}
  HXLDPolyArray(Hkey *key, long num);
  HXLDPolyArray(const Hobject &obj);
  HXLDPolyArray(const HXLDPoly &poly);
  HXLDPolyArray(const HXLD &xld);
  HXLDPolyArray(const HXLDPolyArray &arr);
  HXLDPolyArray(const HXLDArray &arr);
  ~HXLDPolyArray(void) {num_xld_poly_arrays--;}

  HXLDPolyArray &operator= (const HXLDPolyArray &arr);
  HXLDPolyArray &operator= (const HXLDArray &arr);

  // Tools
  virtual const char *HClassName(void)    const;
  virtual const char *ClassName(void)     const;
  virtual const char *InstClassName(void) const;
  virtual       INT4  ClassDBID(void)     const {return XLD_POLYGON_ID;}
  virtual       Hkey  Key(long i)         const {return array[i].Key();}
  virtual    Hobject  ElemId(long i)      const {return array[i].Id();}

  // Access and Manipulation
  HXLDPoly  const   &operator[] (long index) const;
  HXLDPoly          &operator[] (long index);
  HXLDPolyArray      operator() (long min, long max) const;
  HXLDPolyArray     &Append(const HXLDPoly &poly);
  HXLDPolyArray     &Append(const HXLD &xld);
  HXLDPolyArray     &Append(const HXLDPolyArray &polys);
  HXLDPolyArray     &Append(const HXLDArray &xlds);
  virtual void       SetKeys(Hkey *key, long num);
  void               CopyHXLDPolyArray(const HXLDPolyArray &in);
  void               CopyHXLDArray(const HXLDArray &in);

  // Class Functions
  static long NumInst(void) {return num_xld_poly_arrays;}

private:
  static long   num_xld_poly_arrays; 
  void          NewLength(long new_num, long old_num);

public:
  static HXLDPolyArray ReadPolygonXldArcInfo(const HTuple &FileName);
  static HXLDPolyArray ReadPolygonXldArcInfo(const char *FileName);
  virtual void WritePolygonXldArcInfo(const HTuple &FileName) const;
  virtual void WritePolygonXldArcInfo(const char *FileName) const;
  // Combine road hypotheses from two resolution levels.
  virtual HXLDPolyArray CombineRoadsXld(const HXLDModParaArray &ModParallels, const HXLDExtParaArray &ExtParallels, const HXLDPolyArray &CenterLines, const HTuple &MaxAngleParallel, const HTuple &MaxAngleColinear, const HTuple &MaxDistanceParallel, const HTuple &MaxDistanceColinear) const;
  virtual HXLDPolyArray CombineRoadsXld(const HXLDModParaArray &ModParallels, const HXLDExtParaArray &ExtParallels, const HXLDPolyArray &CenterLines, double MaxAngleParallel, double MaxAngleColinear, double MaxDistanceParallel, double MaxDistanceColinear) const;
  // Extract parallel XLD polygons.
  virtual HXLDParaArray GenParallelsXld(const HTuple &Len, const HTuple &Dist, const HTuple &Alpha, const HTuple &Merge) const;
  virtual HXLDParaArray GenParallelsXld(double Len, double Dist, double Alpha, const char *Merge) const;
  virtual HTuple GetLinesXld(HTuple *BeginCol, HTuple *EndRow, HTuple *EndCol, HTuple *Length, HTuple *Phi) const;
  virtual HXLDContArray SplitContoursXld(const HTuple &Mode, const HTuple &Weight, const HTuple &Smooth) const;
  virtual HXLDContArray SplitContoursXld(const char *Mode, long Weight, long Smooth) const;
  virtual HXLDPolyArray AffineTransPolygonXld(const HTuple &HomMat2D) const;
  virtual HXLDPolyArray AffineTransPolygonXld(double HomMat2D) const;
};

#endif
